package app.mar.activities;

import android.app.Fragment;
import android.content.Context;
import android.net.Uri;
import android.os.Bundle;
import android.support.annotation.CallSuper;
import android.util.Log;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import java.util.ArrayList;

import app.mar.utils.game.Game;
import app.mar.utils.game.challenges.Challenge;
import app.mar.utils.game.challenges.GeoIndice;
import app.mar.utils.game.challenges.Indice;
import app.mar.utils.game.challenges.TextIndice;

public abstract class ChallengeFragment extends Fragment {
    public abstract boolean check();
    protected abstract void onInitChallenge(Challenge c);
    protected abstract  void onResetUi();



    protected LinearLayout  mUITextIndices;
    protected TextView      mUIGeoIndice;
    protected LinearLayout  mUIGlobalIndice;

    protected Challenge      mChallenge;


    public void resetUi()
    {
        if(mUIGeoIndice!=null)
        {
            mUIGeoIndice.setVisibility(View.GONE);
        }
        if(mUITextIndices!=null)
        {
            mUITextIndices.removeAllViews();
        }
        if(mUIGlobalIndice!=null)
        {
            mUIGlobalIndice.setVisibility(View.GONE);
        }
        onResetUi();
    }

    private void updateIndices()
    {
        ArrayList<Indice> inds = mChallenge.getIndices();
        if(mUIGlobalIndice!=null && inds.size()>0) mUIGlobalIndice.setVisibility(View.VISIBLE);
        for(int i=0; i<inds.size(); i++)
        {
            Indice ind = inds.get(i);
            if(ind instanceof GeoIndice && mUIGeoIndice!=null)
            {
                mUIGeoIndice.setVisibility(View.VISIBLE);
            }else if(ind instanceof TextIndice && mUITextIndices!=null)
            {
                TextView tv = new TextView(getActivity());
                tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14);
                tv.setText(((TextIndice) ind).getText());
            }
        }
    }

    public Indice showIndice()
    {
        Indice ind = mChallenge.nextIndice();
        if(ind!=null)
        {
            if(getActivity() instanceof  ChallengeDriver)
            {
                if(ind instanceof GeoIndice)
                {
                    if(mUIGeoIndice!=null)
                    {
                        mUIGeoIndice.setVisibility(View.VISIBLE);
                        ind.discover();
                        return ind;
                    }
                    return null;
                }else if(ind instanceof TextIndice)
                {
                    if(mUITextIndices!=null)
                    {
                        TextView tv = new TextView(getActivity());
                        tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14);
                        tv.setText(((TextIndice) ind).getText());
                        ind.discover();
                        return ind;
                    }
                }
            }
        }
        return null;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.e("______", "Calling onCreate ______________ : "+getClass().toString());
    }

    public View onCreateView(LayoutInflater inflater, ViewGroup container,int res) {
        // Inflate the layout for this fragment
        View v = inflater.inflate(res, container, false);
        Log.e("______________", "super.onCreateView: "+mUIGlobalIndice);
        mUITextIndices= (LinearLayout) v.findViewById(R.id.ll_textIndices);
        mUIGeoIndice= (TextView) v.findViewById(R.id.tv_geoIndice);
        mUIGlobalIndice = (LinearLayout) v.findViewById(R.id.ll_rootIndices);
        Log.e("______________", "Ici: "+mUIGlobalIndice);
        Log.e("#########", "setFragment Commit : false");
        return v;
    }


    public void initChallenge(Challenge c)
    {
        Log.e("#########", "initChallenges");
        mChallenge=c;
        updateIndices();
        onInitChallenge(c);
    }

    public void onStart() {
        super.onStart();
        Log.e("---------", "OnStart !");
    }

    public boolean onKeyDown(int keyCode, KeyEvent event) {
        return false;
    }

    private OnFragmentInteractionListener mListener;

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    // TODO: Rename method, update argument and hook method into UI event
    public void onButtonPressed(Uri uri) {
        if (mListener != null) {
            mListener.onFragmentInteraction(uri);
        }
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }

        Log.e("______", "Calling class ______________ : "+getClass().toString());
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    public Challenge getChallenge() {
        return mChallenge;
    }
}
